import { ObjectId } from "mongodb";
import BuildingManager from "./BuildingManager";
import { getPlanetById } from "../../db/planets";
import ResourceManager from "./ResourceManager";
import User from "../User";
import ShipManager from "./ShipManager";
import { System } from "./LocationManager";

export type Planet = {
    _id: ObjectId;
    manager: PlanetManager;
    name: string;
    fields: number;
    resources: ResourceManager;
    buildings: BuildingManager;
    ships: ShipManager;
}

export default class PlanetManager {
    planets: Array<Planet> = [];
    owner: User;
    system!: System;

    constructor(user: User) {
        this.owner = user;
    }

    async fillData(system: System, planets: ObjectId[]) {
        this.system = system;
        await Promise.all(planets.map(async planetId => {
            const planet = await getPlanetById(planetId);

            const planetObject: Planet = {
                _id: planet._id,
                manager: this,
                name: planet.name,
                fields: planet.fields,
                //@ts-ignore
                resources: null,
                //@ts-ignore
                buildings: null,
                //@ts-ignore
                ships: null
            }

            planetObject.resources = await new ResourceManager(planetObject).init(planet.resources);
            planetObject.buildings = await new BuildingManager(planetObject).init(planet.buildings);
            planetObject.ships = await new ShipManager(planetObject).init(planet.ships);

            this.planets.push(planetObject);
        }));
    }

    getPlanetById(id: ObjectId) {
        return this.planets.find(planet => planet._id.equals(id));
    }
}